<?php
/**
 * @copyright	Copyright (C) 2011 Simplify Your Web, Inc. All rights reserved.
 * @license		GNU General Public License version 3 or later; see LICENSE.txt
 */

// No direct access to this file
defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Installer\Installer;
use Joomla\CMS\Installer\InstallerHelper;

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

/**
 * Script file for the Weblink Logos Pro free module package
 */
class pkg_weblinklogosInstallerScript
{
	static $version = '3.1.0';
	static $minimum_needed_library_version = '1.5.3';
	static $available_languages = array('de-DE', 'en-GB', 'fa-IR', 'fr-FR', 'nl-NL', 'pl-PL', 'pt-BR', 'sl-SI', 'ru-RU', 'tr-TR');
	static $download_link = 'https://simplifyyourweb.com/downloads/syw-extension-library';
	static $weblinks_download_link = 'http://extensions.joomla.org/extensions/extension/official-extensions/weblinks';
	static $changelog_link = 'https://simplifyyourweb.com/free-products/weblink-logos/file/365-weblink-logos';
	static $translation_link = 'https://simplifyyourweb.com/translators';

	/**
	 * Called before an install/update method
	 *
	 * @return  boolean  True on success
	 */
	public function preflight($type, $parent)
	{
	    // make sure we are under Joomla 3.8 or over

		if (version_compare(JVERSION, '3.8.0', 'lt') || version_compare(JVERSION, '3.11.0', 'gt')) {
			// keep JFactory and JText so that if installed in Joomla < 3.8, it does not crash but shows the message
			JFactory::getApplication()->enqueueMessage(JText::sprintf('JOOMLA_REQUIRED_VERSION', '3.8', 'https://simplifyyourweb.com/free-products/weblink-logos#downloads'), 'error');
			return false;
		}

	    if ($type == 'install' || $type == 'update') {

    		// check if Weblinks component is present

    		if (!\JFolder::exists(JPATH_ROOT.'/components/com_weblinks')) {

    			$message = Text::_('PKG_WEBLINKLOGOS_MISSING_WEBLINKSCOMPONENT').'.<br /><a href="'.self::$weblinks_download_link.'" target="_blank">'.Text::_('PKG_WEBLINKLOGOS_DOWNLOAD_WEBLINKSCOMPONENT').'</a>.';
    			Factory::getApplication()->enqueueMessage($message, 'error');
    			return false;
    		}

    		// check if syw library is present

    		if (!\JFolder::exists(JPATH_ROOT.'/libraries/syw')) {

    			if (!$this->installOrUpdatePackage($parent, 'lib_syw')) {
    				$message = Text::_('SYWLIBRARY_INSTALLFAILED').'<br /><a href="'.self::$download_link.'" target="_blank">'.Text::_('SYWLIBRARY_DOWNLOAD').'</a>';
    				Factory::getApplication()->enqueueMessage($message, 'error');
    				return false;
    			}

    			Factory::getApplication()->enqueueMessage(Text::sprintf('SYWLIBRARY_INSTALLED', self::$minimum_needed_library_version), 'message');

    		} else {
    			jimport('syw.version');

    			if (SYWVersion::isCompatible(self::$minimum_needed_library_version)) {

    				//Factory::getApplication()->enqueueMessage(Text::_('SYWLIBRARY_COMPATIBLE'), 'message');

    			} else {

    				if (!$this->installOrUpdatePackage($parent, 'lib_syw')) {
    					$message = Text::_('SYWLIBRARY_UPDATEFAILED').'<br />'.Text::_('SYWLIBRARY_UPDATE');
    					Factory::getApplication()->enqueueMessage($message, 'error');
    					return false;
    				}

    				Factory::getApplication()->enqueueMessage(Text::sprintf('SYWLIBRARY_UPDATED', self::$minimum_needed_library_version), 'message');
    			}
    		}
	    }

		return true;
	}

	/**
	 * Called after an install/update method
	 *
	 * @return  boolean  True on success
	 */
	public function postflight($type, $parent)
	{
	    if ($type == 'install' || $type == 'update') {

    		//echo '<p style="margin: 10px 0 20px 0">';
    		//echo '<img src="../modules/mod_weblinklogo/images/logo.png" />';
    		//echo '<br /><br /><span class="label">'.Text::sprintf('PKG_WEBLINKLOGOS_VERSION', self::$version).'</span>';
    		//echo '<br /><br />Olivier Buisard @ <a href="https://simplifyyourweb.com" target="_blank">Simplify Your Web</a>';
    		//echo '</p>';

     		// language test

     		$current_language = Factory::getLanguage()->getTag();
     		if (!in_array($current_language, self::$available_languages)) {
     			Factory::getApplication()->enqueueMessage('The ' . Factory::getLanguage()->getName() . ' language is missing for this extension.<br /><a href="' . self::$translation_link . '" target="_blank">Please consider contributing to its translation</a>', 'notice');
     		}

     		// link to Quickstart

     		$message = Text::sprintf('PKG_WEBLINKLOGOS_INFO_LEARN', 'https://simplifyyourweb.com/documentation/weblink-logos/quickstart-guide');
     		$message .= '<br /><br /><a href="https://simplifyyourweb.com/documentation/weblink-logos/quickstart-guide" target="_blank"><img src="../modules/mod_weblinklogo/images/quickstart.png" /></a>';

     		//Factory::getApplication()->enqueueMessage($message, 'notice');

     		// move default place-holder to /images

     		$imagefiles = array();
     		$imagefiles[] = 'placeholder_camera.png';

     		$media_params = ComponentHelper::getParams('com_media');
     		$images_path = $media_params->get('image_path', 'images');

     		foreach ($imagefiles as $imagefile) {
     		    $src = JPATH_ROOT.'/modules/mod_weblinklogo/images/'.$imagefile;
     		    $dest = JPATH_ROOT.'/'.$images_path.'/'.$imagefile;

     		    if (!\JFile::copy($src, $dest)) {
     		        Factory::getApplication()->enqueueMessage(Text::sprintf('PKG_WEBLINKLOGOS_WARNING_COULDNOTCOPYFILE', $imagefile), 'warning');
     		    }
     		}

     		// remove the old module update site for when it was not packaged

     		$this->removeUpdateSite('module', 'mod_weblinklogo');
     		$this->removeUpdateSite('package', 'pkg_weblinklogos', '', 'http://www.barejoomlatemplates.com/autoupdates/weblinklogos/weblinklogos-update.xml');
     		$this->removeUpdateSite('package', 'pkg_weblinklogos', '', 'http://www.barejoomlatemplates.com/autoupdates/weblinklogos/weblinklogos-pkg-update.xml');
	    }

 		// won't be an update on package first install, when moving from module to package

 		if ($type == 'update') {

			// update warning

			//Factory::getApplication()->enqueueMessage(Text::sprintf('PKG_WEBLINKLOGOS_WARNING_RELEASENOTES', self::$changelog_link), 'warning');

			// delete unnecessary files

			$files = array();

			$files[] = '/modules/mod_weblinklogo/images/normal.png';
			$files[] = '/modules/mod_weblinklogo/images/grow.png';
			$files[] = '/modules/mod_weblinklogo/images/shrink.png';
			$files[] = '/modules/mod_weblinklogo/images/preview.png';
			$files[] = '/modules/mod_weblinklogo/stylemaster.css.php';
			$files[] = '/modules/mod_weblinklogo/style.css.php';
			$files[] = '/modules/mod_weblinklogo/styles/stylemaster.css.php';
			$files[] = '/modules/mod_weblinklogo/fields/weblinkstest.php';
			$files[] = '/modules/mod_weblinklogo/fields/extensionlinkwrapper.php';
			$files[] = '/modules/mod_weblinklogo/fields/extensionlinkswrapper.php';
			$files[] = '/modules/mod_weblinklogo/fields/messagewrapper.php';
			$files[] = '/modules/mod_weblinklogo/fields/sywextensionpresencetestwrapper.php';

			foreach ($files as $file) {
				if (\JFile::exists(JPATH_ROOT.$file) && !\JFile::delete(JPATH_ROOT.$file)) {
					Factory::getApplication()->enqueueMessage(Text::sprintf('PKG_WEBLINKLOGOS_ERROR_DELETINGFILEFOLDER', $file), 'warning');
				}
			}

			// remove old cached headers which may interfere with fixes, updates or new additions

			$filenames_to_delete = array();

			if (function_exists('glob')) {

				$filenames = glob(JPATH_SITE.'/cache/mod_weblinklogos/style_*.css');
				if ($filenames != false) {
					$filenames_to_delete = array_merge($filenames_to_delete, $filenames);
				}

				$filenames = glob(JPATH_SITE.'/cache/mod_weblinklogos/animation_*.js');
				if ($filenames != false) {
					$filenames_to_delete = array_merge($filenames_to_delete, $filenames);
				}

				// from previous versions

				$filenames = glob(JPATH_ROOT.'/modules/mod_weblinklogo/style_*.css');
				if ($filenames != false) {
					$filenames_to_delete = array_merge($filenames_to_delete, $filenames);
				}

				$filenames = glob(JPATH_ROOT.'/modules/mod_weblinklogo/styles/style_*.css');
				if ($filenames != false) {
					$filenames_to_delete = array_merge($filenames_to_delete, $filenames);
				}

				$filenames = glob(JPATH_ROOT.'/modules/mod_weblinklogo/animation_*.js');
				if ($filenames != false) {
					$filenames_to_delete = array_merge($filenames_to_delete, $filenames);
				}
			}

			foreach ($filenames_to_delete as $filename) {
				if (\JFile::exists($filename) && !\JFile::delete($filename)) {
					Factory::getApplication()->enqueueMessage(Text::sprintf('PKG_WEBLINKLOGOS_ERROR_DELETINGFILEFOLDER', $filename), 'warning');
				}
			}

			// overrides warning

			$defaultemplate = $this->getDefaultTemplate();

			if ($defaultemplate) {
				$overrides_path = JPATH_ROOT.'/templates/'.$defaultemplate.'/html/';

				if (\JFolder::exists($overrides_path.'mod_weblinklogo')) {
					Factory::getApplication()->enqueueMessage(Text::_('PKG_WEBLINKLOGOS_WARNING_OVERRIDES'), 'warning');
				}
			}
		}

		return true;
	}

	private function getDefaultTemplate()
	{
		$db = Factory::getDBO();

		$query = $db->getQuery(true);

		$query->select('template');
		$query->from('#__template_styles');
		$query->where($db->quoteName('client_id').'= 0');
		$query->where($db->quoteName('home').'= 1');

		$db->setQuery($query);

		$defaultemplate = '';

		try {
			$defaultemplate = $db->loadResult();
		} catch (\RuntimeException $e) {
			Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
		}

		return $defaultemplate;
	}

	private function removeUpdateSite($type, $element, $folder = '', $location = '')
	{
	    $db = Factory::getDBO();

	    $query = $db->getQuery(true);

	    $query->select('extension_id');
	    $query->from('#__extensions');
	    $query->where($db->quoteName('type').'='.$db->quote($type));
	    $query->where($db->quoteName('element').'='.$db->quote($element));
	    if ($folder) {
	        $query->where($db->quoteName('folder').'='.$db->quote($folder));
	    }

	    $db->setQuery($query);

	    $extension_id = '';
	    try {
	        $extension_id = $db->loadResult();
	    } catch (\RuntimeException $e) {
	        Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
	        return false;
	    }

	    if ($extension_id) {

	        $query->clear();

	        $query->select('update_site_id');
	        $query->from('#__update_sites_extensions');
	        $query->where($db->quoteName('extension_id').'='.$db->quote($extension_id));

	        $db->setQuery($query);

	        $updatesite_id = array(); // can have several results
	        try {
	            $updatesite_id = $db->loadColumn();
	        } catch (\RuntimeException $e) {
	            Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
	            return false;
	        }

	        if (empty($updatesite_id)) {
	            return false;
	        } else if (count($updatesite_id) == 1) {

	            $query->clear();

	            $query->delete($db->quoteName('#__update_sites'));
	            $query->where($db->quoteName('update_site_id').' = '.$db->quote($updatesite_id[0]));

	            $db->setQuery($query);

	            try {
	                $db->execute();
	            } catch (\RuntimeException $e) {
	                Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
	                return false;
	            }
	        } else { // several update sites exist for the same extension therefore we need to specify which to delete

	            if ($location) {
	                $query->clear();

	                $query->delete($db->quoteName('#__update_sites'));
	                $query->where($db->quoteName('update_site_id').' IN ('.implode(',', $updatesite_id).')');
	                $query->where($db->quoteName('location').' = '.$db->quote($location));

	                $db->setQuery($query);

	                try {
	                    $db->execute();
	                } catch (\RuntimeException $e) {
	                    Factory::getApplication()->enqueueMessage(Text::_('JERROR_AN_ERROR_HAS_OCCURRED'), 'error');
	                    return false;
	                }
	            } else {
	                return false;
	            }
	        }
	    } else {
	        return false;
	    }

	    return true;
	}

	private function installOrUpdatePackage($parent, $package_name, $installation_type = 'install')
	{
		// Get the path to the package

		$sourcePath = $parent->getParent()->getPath('source');
		$sourcePackage = $sourcePath . '/packages/'.$package_name.'.zip';

		// Extract and install the package

		$package = InstallerHelper::unpack($sourcePackage);
		$tmpInstaller = new Installer;

		try {
			if ($installation_type == 'install') {
				$installResult = $tmpInstaller->install($package['dir']);
			} else {
				$installResult = $tmpInstaller->update($package['dir']);
			}
		} catch (\Exception $e) {
			return false;
		}

		return true;
	}

	/**
	 * Called on installation
	 *
	 * @return  boolean  True on success
	 */
	public function install($parent) { }

	/**
	 * Called on update
	 *
	 * @return  boolean  True on success
	 */
	public function update($parent) { }

	/**
	 * Called on uninstallation
	 */
	public function uninstall($parent) { }
}
?>